unit main;

interface

uses
  Classes, SysUtils, FileUtil, LR_Shape, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons, LCLType, LCLIntf, mocpi;

{ TForm1 }

type
  TShowState = (ssStation, ssTime, ssTuning, ssVolume);

  TForm1 = class(TForm)
    moreButton: TSpeedButton;
    SelectButton: TBitBtn;
    Display: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Shape2: TShape;
    Shape3: TShape;
    SpeedButton2: TSpeedButton;
    StationIndexLabel: TLabel;
    Memo1: TMemo;
    Shape1: TShape;
    RadioFace: TShape;
    SelectUpDown: TUpDown;
    PumpTimer: TTimer;
    VolumeUpDown: TUpDown;
    SpeedButton1: TSpeedButton;
    VolumeLabel: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure PumpTimerTimer(Sender: TObject);
    procedure SelectButtonClick(Sender: TObject);
    procedure moreButtonClick(Sender: TObject);
    procedure RadioFaceMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RadioFaceMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure RadioFaceMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Shape1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SelectUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure Shape2MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpeedButton2Click(Sender: TObject);
    procedure VolumeUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { private declarations }

    // variables used to drag the form on the screen
    FMoving: boolean;
    FPosX: integer;
    FPosY: integer;


    FCurrentState: string;
    FCurrentURL: string;
    FCurrentTitle: string;

    // radio station variables
    FStationCount: integer;    // number of stations found in JSON table
    FPlayIndex: integer;       // index of currently playing station, -1 none playing
    FCurrentIndex: integer;    // index of currently displayed station

    FCurrentVolume: integer;   // volume reported by MOC

    FCurrentTime: TDateTime;

    FShowState: TShowState;
    FStateStart: TDateTime;

    procedure SetState(value: TShowState);

    procedure MocStateChanged(Sender: TObject; events: TMocEvents; const mocState: TMocState);

    procedure AdjustVolume(delta: integer);
    procedure AdjustStation(delta: integer);
    procedure PlayStation;     // Play station at FCurrentIndex

    procedure ShowPlayingStation;
    procedure ShowCurrentStation;
    procedure ShowCurrentDateTime;
    procedure ShowCurrentVolume;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  strUtils, radios;

const
  SecondsPerDay = 24*60*60;
  ClockShowTime = 3/SecondsPerDay;  // 3 seconds
  StationShowTime = 8/SecondsPerDay;

const
  cNoServer = ' '#$E2#$8F#$80' ';  // ⏀   .
  cStop     = ' '#$E2#$9B#$94' ';  // ⛔  ./
  cPlay     = ' '#$E2#$9A#$9F' ';  // ⚟   .
  cPause    = ' '#$E2#$8F#$B8' ';  // ⏸   .
  //cTuning   = ' '#$E2#$97#$B4' ';  // ◴  .
  cTuning   = ' '#$E2#$AD#$A5' ';  //  ⭥ .
  cVolume1   = #$E2#$96#$88 ;     // █
  cVolume0   = #$E2#$96#$81 ;     // ▁
  //cVolume0   = #$E2#$96#$82 ;     // ▂


{ TForm1 }

procedure TForm1.AdjustVolume(delta: integer);
const
  STEP = 5;
var
  s: string;
begin
  delta := STEP*delta;
  setVolume(delta);
  FCurrentVolume := volume;

  // report command
  memo1.lines.add('');
  if delta > 0 then
    s := '+'
  else
    s := '';
  memo1.lines.add(format('[rso] Adjust volume %s%d', [s, delta]));
  // report result
  memo1.lines.add(format('[moc] Volume = %d', [FCurrentVolume]));
  // show on window
  VolumeLabel.Caption := Format('%d/100', [FCurrentVolume]);
end;

procedure TForm1.AdjustStation(delta: integer);
begin
  if FStationCount < 1 then
    exit;
  FCurrentIndex := FCurrentIndex + delta;
  if FCurrentIndex < 0 then
    FCurrentIndex := FStationCount-1
  else if FCurrentIndex >= FStationCount then
    FCurrentIndex := 0;

  StationIndexLabel.caption := Format('%d/%d', [FCurrentIndex+1, FStationCount]);
end;

procedure TForm1.SelectUpDownClick(Sender: TObject; Button: TUDBtnType);
var
  delta: integer;
begin
  if Button = btNext then
     delta := 1
   else
     delta := -1;
  AdjustStation(delta);
  SetState(ssTuning);
end;

procedure TForm1.Shape2MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  WindowState := wsMinimized;
end;

procedure TForm1.SpeedButton2Click(Sender: TObject);
begin
  with Sender as TSpeedButton do begin
    if down then
      pause
    else
      resume;
  end;
  SetState(ssStation);
end;

procedure TForm1.VolumeUpDownClick(Sender: TObject; Button: TUDBtnType);
var
  delta: integer;
begin
  if Button = btNext then
     delta := 1
   else
     delta := -1;
  AdjustVolume(delta);
  SetState(ssVolume);
end;

procedure TForm1.MocStateChanged(Sender: TObject; events: TMocEvents;
            const mocState: TMocState);
begin
  Memo1.Lines.Add('');
  if meState in Events then
    Memo1.Lines.add('[moc] state: ' + mocState.state);
  if meUri in Events then
    Memo1.Lines.add('[moc] url: ' + mocState.uri);
  if meTitle in Events then
    Memo1.Lines.add('[moc] title: "' + mocState.title + '"');

  if meState in Events then
    FCurrentState := mocState.state;
  if meUri in Events then
    FCurrentURL := mocState.uri;
  if meTitle in Events then
    FCurrentTitle := mocState.title;

  if (meState in Events) or (meUri in Events) then
    ShowPlayingStation;
end;

procedure TForm1.PlayStation;
var
  urls: TUrlList;
begin
  Memo1.Lines.Add('');
  Memo1.Lines.Add(Format('[rso] play station #%d: %s', [FCurrentIndex, radioList[FCurrentIndex]]));

  if getUrlList(FCurrentIndex, urls) > 0 then
    playwt(urls)
  else begin
    Display.Text := 'No URLS for station';
    Memo1.Lines.add('[rso] No URLS for station');
  end;
end;

procedure TForm1.SetState(value: TShowState);
begin
  FStateStart := now;
  FShowState := value;
  case value of
    ssStation: ShowPlayingStation;
    ssTime:    ShowCurrentDateTime;
    ssTuning:  ShowCurrentStation;
    ssVolume:  ShowCurrentVolume;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  url: string;
begin
  FPlayIndex := -1;
  moreButtonClick(moreButton);
  repeat
    try
      loadRadioJson;
      if not assigned(radioList) then
        FStationCount := 0
      else
        FStationCount := radioList.count;
      if (state = 'PLAY') then begin
        FCurrentIndex := urlToIndex(uri);
        if (FCurrentIndex < 0) then
          Display.Text := 'Unknown Station'
        else
          AdjustStation(0)
      end
      else if FStationCount = 0 then
        Display.Text := 'No stations in radio.json'
      else begin
        FCurrentIndex := 0;
        AdjustStation(0);
      end;
      VolumeLabel.Caption := Format('%d/100', [volume]);
      MocPoller.AddListener(@MocStateChanged);
      MocPoller.Resume;
      SetState(ssStation);
      exit;
    except
       on e: exception do begin
         if MessageDlg('JSON error: ' + e.message,
           mtError, [mbAbort, mbRetry], 0) = mrAbort then
             halt(1);
       end;
    end;
  until false;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  MocPoller.RemoveListener(@MocStateChanged);
end;

  { Truncates seconds and millisec out of TDateTimeValues.}
function TruncDateTimeToMins(const DateTime: TDateTime): TDateTime;
var
  ts: TTimeStamp;
begin
  { Call DateTimeToTimeStamp to convert to TimeStamp: }
  ts := SysUtils.DateTimeToTimeStamp(DateTime);
  { Truncate time by removing the seconds and milliseconds}
  ts.Time := (ts.Time div (60*1000))*60*1000;
  { Convert back to TDateTime: }
  Result := SysUtils.TimeStampToDateTime(ts);
end;

  { Truncates milliseconds out of TDateTimeValues.}
function TruncDateTimeToSecs(const DateTime: TDateTime): TDateTime;
var
  ts: TTimeStamp;
begin
  { Call DateTimeToTimeStamp to convert to TimeStamp: }
  ts := SysUtils.DateTimeToTimeStamp(DateTime);
  { Truncate time by removing the milliseconds}
  ts.Time := (ts.Time div 1000)*1000;
  { Convert back to TDateTime: }
  result := SysUtils.TimeStampToDateTime(ts);
end;


procedure TForm1.PumpTimerTimer(Sender: TObject);
var
  cdt: TDateTime;
begin
  cdt := TruncDateTimeToMins(now);
  if cdt <> FCurrentTime then begin
    FCurrentTime := cdt;
    if FShowState = ssTime then
      ShowCurrentDateTime;
  end;

  if FShowState <> ssStation then begin
    if now - FStateStart > ClockShowTime then
      SetState(ssStation)
  end
  else {if FShowState <> ssTime then} begin
    if now - FStateStart > StationShowTime then
      SetState(ssTime)
  end;
end;


procedure TForm1.ShowCurrentDateTime;
begin
  Display.Text := FormatDateTime('dddd yyyy-mm-dd, hh:mm', FCurrentTime);
end;


procedure TForm1.ShowPlayingStation;
var
  s: string;
  i: integer;
begin
  if FCurrentState = 'NO SERVER' then
    s := cNoServer
  else if (FCurrentState = 'PLAY') or (FCurrentState = 'PAUSE') then begin
    s := '';
    i := urlToIndex(FCurrentUrl);
    if i >= 0 then begin
      FPlayIndex := i;
      FCurrentIndex := i;
      s := RadioList[i];
    end;
    if FCurrentState = 'PAUSE' then
      s := cPause + s
    else
      s := cPlay + s;
  end
  else {if FCurrentState = 'STOP' then }
    s := cStop;
  Display.Text := s;
end;


procedure TForm1.ShowCurrentStation;
var
  s: string;
begin
  if (0 <= FCurrentIndex) and (FCurrentIndex < FStationCount) then
    s := cTuning + radioList[FCurrentIndex]
  else
    s := '';
  Display.Text := s;
end;


procedure TForm1.ShowCurrentVolume;
const
  SZ = 39;
var
  n: integer;
begin
  n := round(SZ*FCurrentVolume/100);
  Display.Text := DupeString(cVolume1, n) + DupeString(cVolume0, SZ - n);
end;


procedure TForm1.SelectButtonClick(Sender: TObject);
begin
  PlayStation;
  SetState(ssStation);
end;

procedure TForm1.RadioFaceMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FPosX := X;
  FPosY := Y;
  FMoving := true;
end;

procedure TForm1.RadioFaceMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FMoving then
    SetBounds(Left + X - FPosX, Top + Y - FPosY, Width, Height)
end;

procedure TForm1.RadioFaceMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FMoving := false;
end;

procedure TForm1.Shape1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  stop;
  close
end;


procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  MessageDlg(info,  mtInformation, [mbOk], 0);
end;

procedure TForm1.moreButtonClick(Sender: TObject);
var
  newheight: integer;
begin
  with Sender as TSpeedButton do begin
      if down then begin
        font.color := clRed;
        newheight := 500;
      end
      else begin
        font.color := clDefault;
        newheight := memo1.top;
      end;
  end;
  setbounds(left, top, width, newheight);
end;


end.

